@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -17,17 +17,24 @@
 #include "jalv_internal.h"
 
 #include "lv2/core/attributes.h"
+#include "suil/suil.h"
+#include "zix/sem.h"
 
 LV2_DISABLE_DEPRECATION_WARNINGS
+#include <glib.h>
 #include <gtkmm/button.h>
 #include <gtkmm/main.h>
+#include <gtkmm/object.h>
+#include <gtkmm/widget.h>
 #include <gtkmm/window.h>
 LV2_RESTORE_WARNINGS
 
-Gtk::Main* jalv_gtk_main = NULL;
+#include <cstdint>
+
+static Gtk::Main* jalv_gtk_main = nullptr;
 
 int
-jalv_init(int* argc, char*** argv, JalvOptions* opts)
+jalv_init(int* argc, char*** argv, JalvOptions*)
 {
 	jalv_gtk_main = new Gtk::Main(*argc, *argv);
 	return 0;
@@ -53,13 +60,13 @@ jalv_ui_port_event(Jalv*       jalv,
 }
 
 bool
-jalv_discover_ui(Jalv* jalv)
+jalv_discover_ui(Jalv*)
 {
 	return true;
 }
 
 float
-jalv_ui_refresh_rate(Jalv* jalv)
+jalv_ui_refresh_rate(Jalv*)
 {
 	return 30.0f;
 }
@@ -70,23 +77,27 @@ jalv_open_ui(Jalv* jalv)
 	Gtk::Window* window = new Gtk::Window();
 
 	if (jalv->ui) {
-		jalv_ui_instantiate(jalv, jalv_native_ui_type(), NULL);
+		jalv_ui_instantiate(jalv, jalv_native_ui_type(), nullptr);
 	}
 
 	if (jalv->ui_instance) {
-		GtkWidget* widget = (GtkWidget*)suil_instance_get_widget(
-			jalv->ui_instance);
+		GtkWidget* widget = static_cast<GtkWidget*>(
+		    suil_instance_get_widget(jalv->ui_instance));
+
 		Gtk::Widget* widgetmm = Glib::wrap(widget);
 		window->add(*Gtk::manage(widgetmm));
 		widgetmm->show_all();
 
 		g_timeout_add(1000 / jalv->ui_update_hz,
-		              (GSourceFunc)jalv_update, jalv);
+		              reinterpret_cast<GSourceFunc>(jalv_update),
+		              jalv);
 	} else {
 		Gtk::Button* button = Gtk::manage(new Gtk::Button("Close"));
 		window->add(*Gtk::manage(button));
 	}
 
+	jalv_init_ui(jalv);
+
 	window->set_resizable(jalv_ui_is_resizable(jalv));
 	window->show_all();
 
@@ -100,7 +111,7 @@ jalv_open_ui(Jalv* jalv)
 }
 
 int
-jalv_close_ui(Jalv* jalv)
+jalv_close_ui(Jalv*)
 {
 	Gtk::Main::quit();
 	return 0;
