@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2012 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -16,6 +16,13 @@
 
 #include "jalv_internal.h"
 
+#include "lv2/log/log.h"
+#include "lv2/urid/urid.h"
+
+#include <stdarg.h>
+#include <stdbool.h>
+#include <stdio.h>
+
 int
 jalv_printf(LV2_Log_Handle handle,
             LV2_URID       type,
@@ -35,9 +42,26 @@ jalv_vprintf(LV2_Log_Handle handle,
              va_list        ap)
 {
 	// TODO: Lock
-	Jalv* jalv = (Jalv*)handle;
-	if (type == jalv->urids.log_Trace && !jalv->opts.dump) {
-		return 0;
+	Jalv* jalv  = (Jalv*)handle;
+	bool  fancy = true;
+	if (type == jalv->urids.log_Trace && jalv->opts.trace) {
+		jalv_ansi_start(stderr, 32);
+		fprintf(stderr, "trace: ");
+	} else if (type == jalv->urids.log_Error) {
+		jalv_ansi_start(stderr, 31);
+		fprintf(stderr, "error: ");
+	} else if (type == jalv->urids.log_Warning) {
+		jalv_ansi_start(stderr, 33);
+		fprintf(stderr, "warning: ");
+	} else {
+		fancy = false;
 	}
-	return vfprintf(stderr, fmt, ap);
+
+	const int st = vfprintf(stderr, fmt, ap);
+
+	if (fancy) {
+		jalv_ansi_reset(stderr);
+	}
+
+	return st;
 }
